# Copyright (C) 2010 by Aaron Bentley
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import datetime
import os.path
from subprocess import call
import sys
import webbrowser

from bzrlib import branch
from bzrlib import export
from bzrlib.plugins.launchpad import lp_api
from bzrlib import osutils
from configobj import ConfigObj
from launchpadlib.launchpad import Launchpad
from launchpadlib.uris import LPNET_SERVICE_ROOT, STAGING_SERVICE_ROOT

RELEASE_DIR = '/home/abentley/releases'


class UserError(Exception):
    pass


class LaunchpadBranch(lp_api.LaunchpadBranch):

    @staticmethod
    def tweak_url(url, launchpad):
        """Adjust a URL to work with staging, if needed."""
        if not str(launchpad._root_uri).startswith(STAGING_SERVICE_ROOT):
            return url
        if url is None:
            return None
        return url.replace('bazaar.launchpad.net',
                           'bazaar.staging.launchpad.net')

class TheKraken(object):

    @staticmethod
    def get_config():
        CONFIG_PATH = '.the_kraken'
        if not os.path.exists(CONFIG_PATH):
            raise UserError('Config file "%s" not found' % CONFIG_PATH)
        return ConfigObj(CONFIG_PATH)

    def __init__(self, location, version, config):
        self.location = location
        self.version = version
        self.config = config
        self.testing = config.as_bool('testing')
        self.series_name = config.get('series')

    def get_branch(self):
        if self.testing:
            root = STAGING_SERVICE_ROOT
        else:
            root = LPNET_SERVICE_ROOT
        launchpad = Launchpad.login_with('The Kraken', root)
        bzr_branch = branch.Branch.open(self.location)
        return LaunchpadBranch.from_bzr(launchpad, bzr_branch)

    def get_series(self, lpb):
        if self.series_name is not None:
            for series in lpb.lp.project.series:
                if series.name == self.series_name:
                    return series
            else:
                raise UserError('No series named %s' % self.series_name)

        series = [s for s in lpb.lp.project.series if s.branch == lpb.lp]
        if len(series) == 0:
            raise UserError('No series linked to branch "%s".' %
                            lpb.lp.bzr_identity)
        if len(series) > 1:
            raise UserError('Multiple series linked to branch "%s".' %
                            lpb.lp.bzr_identity)
        return series[0]

    def upload_tarball(self, release, tarball_path, tarball_name):
        with open(tarball_path) as tarball:
            tarball_content = tarball.read()
        with open(tarball_path + '.sig') as sig:
            sig_content = sig.read()
        release.add_file(file_type='Code Release Tarball',
            file_content=tarball_content, filename=tarball_name,
            signature_content=sig_content,
            signature_filename=tarball_name + '.sig',
            content_type="application/x-gzip; charset=binary")

    def run(self):
        lpb = self.get_branch()
        series = self.get_series(lpb)
        tarball_name = '%s-%s.tar.gz' % (lpb.lp.project.name, self.version)
        if self.testing:
            tarball_name = 'testing-' + tarball_name
        tarball_path = os.path.join(RELEASE_DIR, tarball_name)
        project_name = lpb.lp.project.name

        print "Testing: %s" % self.testing
        print "Project: %s" % project_name
        print " Series: %s" % series.name
        print "Version: %s" % self.version
        print "Tarball: %s" % tarball_path
        if os.path.exists(tarball_path):
            raise UserError('File exists: "%s"' % tarball_path)
        print "Continue? [y/n]"
        ch = osutils.getchar()
        if ch.lower() != 'y':
            sys.exit(0)
        export.export(lpb.bzr.basis_tree(), tarball_path, root=project_name)
        call(['gpg', '--detach-sign', tarball_path])
        milestone = series.newMilestone(name=self.version)
        release = milestone.createProductRelease(
            date_released=datetime.datetime.utcnow())
        self.upload_tarball(release, tarball_path, tarball_name)
        webbrowser.open(self.get_release_url(release))

    @staticmethod
    def get_release_url(release):
        release_url = release.self_link
        release_url = release_url.replace('https://api.', 'https://')
        return release_url.replace('launchpad.net/1.0', 'launchpad.net')
